@@ -25,6 +25,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include <math.h>
 #include <stdint.h>
 #include <assert.h>
@@ -108,7 +110,7 @@ static int get_position(const bam_pileup1_t *p, int *len)
         if ( cig==BAM_CHARD_CLIP ) continue;
         if ( cig==BAM_CPAD ) continue;
         if ( cig==BAM_CREF_SKIP ) continue;
-        fprintf(pysamerr,"todo: cigar %d\n", cig);
+        fprintf(pysam_stderr,"todo: cigar %d\n", cig);
         assert(0);
     }
     *len = n_tot_bases;
@@ -479,7 +481,7 @@ void calc_SegBias(const bcf_callret1_t *bcr, bcf_call_t *call)
     double sum = 0;
     const double log2 = log(2.0);
 
-    // fprintf(pysamerr,"M=%.1f  p=%e q=%e f=%f  dp=%d\n",M,p,q,f,avg_dp);
+    // fprintf(pysam_stderr,"M=%.1f  p=%e q=%e f=%f  dp=%d\n",M,p,q,f,avg_dp);
     int i;
     for (i=0; i<call->n; i++)
     {
@@ -494,7 +496,7 @@ void calc_SegBias(const bcf_callret1_t *bcr, bcf_call_t *call)
         else
             tmp = log(2*f*(1-f)*exp(-q) + f*f*exp(-2*q) + (1-f)*(1-f)) + p;
         sum += tmp;
-        // fprintf(pysamerr,"oi=%d %e\n", oi,tmp);
+        // fprintf(pysam_stderr,"oi=%d %e\n", oi,tmp);
     }
     call->seg_bias = sum;
 }
@@ -658,7 +660,7 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
             }
         }
 
-//      if (ref_base < 0) fprintf(pysamerr, "%d,%d,%f,%d\n", call->n_alleles, x, sum_min, call->unseen);
+//      if (ref_base < 0) fprintf(pysam_stderr, "%d,%d,%f,%d\n", call->n_alleles, x, sum_min, call->unseen);
         call->shift = (int)(sum_min + .499);
     }
     // combine annotations
