@@ -25,6 +25,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include <assert.h>
 #include <ctype.h>
 #include <string.h>
@@ -40,8 +42,6 @@ KSORT_INIT_GENERIC(uint32_t)
 #define MINUS_CONST 0x10000000
 #define INDEL_WINDOW_SIZE 50
 
-extern const char bam_nt16_nt4_table[];
-
 void *bcf_call_add_rg(void *_hash, const char *hdtext, const char *list)
 {
     const char *s, *p, *q, *r, *t;
@@ -201,7 +201,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                 j = bam_cigar2qlen(p->b->core.n_cigar, bam_get_cigar(p->b));
                 if (j > max_rd_len) max_rd_len = j;
             }
-            float frac = (float)na/nt;
+            double frac = (double)na/nt;
             if ( !indel_support_ok && na >= bca->min_support && frac >= bca->min_frac )
                 indel_support_ok = 1;
             if ( na > bca->max_support && frac > 0 ) bca->max_support = na, bca->max_frac = frac;
@@ -219,7 +219,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
             if (aux[i] != aux[i-1]) ++n_types;
         // Taking totals makes it hard to call rare indels
         if ( !bca->per_sample_flt )
-            indel_support_ok = ( (float)n_alt / n_tot < bca->min_frac || n_alt < bca->min_support ) ? 0 : 1;
+            indel_support_ok = ( (double)n_alt / n_tot < bca->min_frac || n_alt < bca->min_support ) ? 0 : 1;
         if ( n_types == 1 || !indel_support_ok ) { // then skip
             free(aux); return -1;
         }
@@ -227,7 +227,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
             free(aux);
             // TODO revisit how/whether to control printing this warning
             if (hts_verbose >= 2)
-                fprintf(pysamerr, "[%s] excessive INDEL alleles at position %d. Skip the position.\n", __func__, pos + 1);
+                fprintf(pysam_stderr, "[%s] excessive INDEL alleles at position %d. Skip the position.\n", __func__, pos + 1);
             return -1;
         }
         types = (int*)calloc(n_types, sizeof(int));
@@ -300,7 +300,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
             if ((double)(max2&0xffff) / ((max2&0xffff) + (max2>>16)) >= 0.7) max2_i = -1;
             if (max_i >= 0) r[max_i] = 15;
             if (max2_i >= 0) r[max2_i] = 15;
-            //for (i = 0; i < right - left; ++i) fputc("=ACMGRSVTWYHKDBN"[(int)r[i]], pysamerr); fputc('\n', pysamerr);
+            //for (i = 0; i < right - left; ++i) fputc("=ACMGRSVTWYHKDBN"[(int)r[i]], pysam_stderr); fputc('\n', pysam_stderr);
         }
         free(ref0); free(cns);
     }
@@ -329,7 +329,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                         if (p->indel == types[t]) {
                             uint8_t *seq = bam_get_seq(p->b);
                             for (k = 1; k <= p->indel; ++k) {
-                                int c = bam_nt16_nt4_table[bam_seqi(seq, p->qpos + k)];
+                                int c = seq_nt16_int[bam_seqi(seq, p->qpos + k)];
                                 assert(c<5);
                                 ++inscns_aux[(t*max_ins+(k-1))*5 + c];
                             }
@@ -368,17 +368,17 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
         else if (types[t] > 0) ir = est_indelreg(pos, ref, types[t], &inscns[t*max_ins]);
         else ir = est_indelreg(pos, ref, -types[t], 0);
         if (ir > bca->indelreg) bca->indelreg = ir;
-//      fprintf(pysamerr, "%d, %d, %d\n", pos, types[t], ir);
+//      fprintf(pysam_stderr, "%d, %d, %d\n", pos, types[t], ir);
         // realignment
         for (s = K = 0; s < n; ++s) {
             // write ref2
             for (k = 0, j = left; j <= pos; ++j)
-                ref2[k++] = bam_nt16_nt4_table[(int)ref_sample[s][j-left]];
+                ref2[k++] = seq_nt16_int[(int)ref_sample[s][j-left]];
             if (types[t] <= 0) j += -types[t];
             else for (l = 0; l < types[t]; ++l)
                      ref2[k++] = inscns[t*max_ins + l];
             for (; j < right && ref[j]; ++j)
-                ref2[k++] = bam_nt16_nt4_table[(int)ref_sample[s][j-left]];
+                ref2[k++] = seq_nt16_int[(int)ref_sample[s][j-left]];
             for (; k < max_ref2; ++k) ref2[k] = 4;
             if (j < right) right = j;
             // align each read to ref2
@@ -402,7 +402,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                 }
                 // write the query sequence
                 for (l = qbeg; l < qend; ++l)
-                    query[l - qbeg] = bam_nt16_nt4_table[bam_seqi(seq, l)];
+                    query[l - qbeg] = seq_nt16_int[bam_seqi(seq, l)];
                 { // do realignment; this is the bottleneck
                     const uint8_t *qual = bam_get_qual(p->b), *bq;
                     uint8_t *qq;
@@ -430,11 +430,11 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                 }
 /*
                 for (l = 0; l < tend - tbeg + abs(types[t]); ++l)
-                    fputc("ACGTN"[(int)ref2[tbeg-left+l]], pysamerr);
-                fputc('\n', pysamerr);
-                for (l = 0; l < qend - qbeg; ++l) fputc("ACGTN"[(int)query[l]], pysamerr);
-                fputc('\n', pysamerr);
-                fprintf(pysamerr, "pos=%d type=%d read=%d:%d name=%s qbeg=%d tbeg=%d score=%d\n", pos, types[t], s, i, bam1_qname(p->b), qbeg, tbeg, sc);
+                    fputc("ACGTN"[(int)ref2[tbeg-left+l]], pysam_stderr);
+                fputc('\n', pysam_stderr);
+                for (l = 0; l < qend - qbeg; ++l) fputc("ACGTN"[(int)query[l]], pysam_stderr);
+                fputc('\n', pysam_stderr);
+                fprintf(pysam_stderr, "pos=%d type=%d read=%d:%d name=%s qbeg=%d tbeg=%d score=%d\n", pos, types[t], s, i, bam1_qname(p->b), qbeg, tbeg, sc);
 */
             }
         }
@@ -490,7 +490,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                 if (seqQ > 255) seqQ = 255;
                 p->aux = (sc[0]&0x3f)<<16 | seqQ<<8 | indelQ; // use 22 bits in total
                 sumq[sc[0]&0x3f] += indelQ < seqQ? indelQ : seqQ;
-//              fprintf(pysamerr, "pos=%d read=%d:%d name=%s call=%d indelQ=%d seqQ=%d\n", pos, s, i, bam1_qname(p->b), types[sc[0]&0x3f], indelQ, seqQ);
+//              fprintf(pysam_stderr, "pos=%d read=%d:%d name=%s call=%d indelQ=%d seqQ=%d\n", pos, s, i, bam1_qname(p->b), types[sc[0]&0x3f], indelQ, seqQ);
             }
         }
         // determine bca->indel_types[] and bca->inscns
@@ -522,7 +522,7 @@ int bcf_call_gap_prep(int n, int *n_plp, bam_pileup1_t **plp, int pos, bcf_calla
                     if (x == bca->indel_types[j]) break;
                 p->aux = j<<16 | (j == 4? 0 : (p->aux&0xffff));
                 if ((p->aux>>16&0x3f) > 0) ++n_alt;
-                //fprintf(pysamerr, "X pos=%d read=%d:%d name=%s call=%d type=%d seqQ=%d indelQ=%d\n", pos, s, i, bam1_qname(p->b), (p->aux>>16)&0x3f, bca->indel_types[(p->aux>>16)&0x3f], (p->aux>>8)&0xff, p->aux&0xff);
+                //fprintf(pysam_stderr, "X pos=%d read=%d:%d name=%s call=%d type=%d seqQ=%d indelQ=%d\n", pos, s, i, bam1_qname(p->b), (p->aux>>16)&0x3f, bca->indel_types[(p->aux>>16)&0x3f], (p->aux>>8)&0xff, p->aux&0xff);
             }
         }
     }
