@@ -3,6 +3,7 @@ import pysam
 import unittest
 import collections
 import copy
+import array
 
 from TestUtils import checkFieldEqual
 
@@ -319,7 +320,7 @@ class TestAlignedSegment(ReadTest):
              (None, 25, 'T'), (None, 26, 'T'),
              (5, 27, 'A'), (6, 28, 'A'), (7, 29, 'A'), (8, 30, 'A')]
             )
-        
+
         a.cigarstring = "5M2D2I2M"
         a.set_tag("MD", "4C^TT2")
         self.assertEqual(
@@ -331,6 +332,34 @@ class TestAlignedSegment(ReadTest):
              (7, 27, 'A'), (8, 28, 'A')]
             )
 
+    def test_get_aligned_pairs_skip_reference(self):
+        a = self.buildRead()
+        a.query_sequence = "A" * 10
+        a.cigarstring = "5M1N5M"
+        a.set_tag("MD", "10")
+
+        self.assertEqual(
+            a.get_aligned_pairs(with_seq=True),
+            [(0, 20, 'A'), (1, 21, 'A'), (2, 22, 'A'),
+             (3, 23, 'A'), (4, 24, 'A'), (None, 25, None),
+             (5, 26, 'A'), (6, 27, 'A'), (7, 28, 'A'),
+             (8, 29, 'A'), (9, 30, 'A')])
+
+        self.assertEqual(
+            a.get_aligned_pairs(with_seq=False),
+            [(0, 20), (1, 21), (2, 22),
+             (3, 23), (4, 24), (None, 25),
+             (5, 26), (6, 27), (7, 28),
+             (8, 29), (9, 30)])
+
+        self.assertEqual(
+            a.get_aligned_pairs(matches_only=True, with_seq=False),
+            [(0, 20), (1, 21),
+             (2, 22), (3, 23),
+             (4, 24), (5, 26),
+             (6, 27), (7, 28),
+             (8, 29), (9, 30)])
+
     def testNoSequence(self):
         '''issue 176: retrieving length without query sequence
         with soft-clipping.
@@ -347,13 +376,60 @@ class TestAlignedSegment(ReadTest):
         self.assertEqual(a.query_alignment_length, 20)
 
 
+class TestCigarStats(ReadTest):
+    
+    def testStats(self):
+        
+        a = self.buildRead()
+
+        a.cigarstring = None
+        self.assertEqual(
+            [list(x) for x in a.get_cigar_stats()],
+            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+
+        a.cigarstring = "10M"
+        self.assertEqual(
+            [list(x) for x in a.get_cigar_stats()],
+            [[10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+             [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+
+        a.cigarstring = "10M2I2M"
+        self.assertEqual(
+            [list(x) for x in a.get_cigar_stats()],
+            [[12, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+             [2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+
+        for i, x in enumerate("MIDNSHP=X"):
+            a.cigarstring = "2{}".format(x)
+            expected = [[0] * 11, [0] * 11]
+            expected[0][i] = 2
+            expected[1][i] = 1
+            self.assertEqual(
+                [list(x) for x in a.get_cigar_stats()],
+                expected)
+
+        a.cigarstring = "10M"
+        a.set_tag("NM", 5)
+        self.assertEqual(
+            [list(x) for x in a.get_cigar_stats()],
+            [[10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5],
+             [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+
+        a.cigarstring = None
+        self.assertEqual(
+            [list(x) for x in a.get_cigar_stats()],
+            [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5],
+             [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]])
+
+
 class TestAlignedPairs(unittest.TestCase):
     filename = os.path.join(DATADIR, "example_aligned_pairs.bam")
 
     def testReferenceBases(self):
         """reference bases should always be the same nucleotide
         """
-        reference_bases = collections.defaultdict(list)        
+        reference_bases = collections.defaultdict(list)
         with pysam.AlignmentFile(self.filename) as inf:
             for c in inf.pileup():
                 for r in c.pileups:
@@ -389,7 +465,25 @@ class TestTags(ReadTest):
         self.assertEqual(False, a.has_tag("NM"))
         # check if deleting a non-existing tag is fine
         a.set_tag("NM", None)
+        a.set_tag("NM", None)
 
+    def testArrayTags(self):
+        read = self.buildRead()
+        supported_dtypes = "bhBHf"
+        unsupported_dtypes = "lLd"
+
+        for dtype in supported_dtypes:
+            key = "F" + dtype
+            read.set_tag(key, array.array(dtype, range(10)))
+            ary = read.get_tag(key)
+
+        for dtype in unsupported_dtypes:
+            key = "F" + dtype
+            self.assertRaises(ValueError,
+                              read.set_tag,
+                              key,
+                              array.array(dtype, range(10)))
+        
     def testAddTagsType(self):
         a = self.buildRead()
         a.tags = None
@@ -551,6 +645,23 @@ class TestTags(ReadTest):
             "A" * 5 + "C" * 3 + "A" * 5,
             a.get_reference_sequence())
 
+    def testMDTagRefSkipping(self):
+        a = self.buildRead()
+
+        a.cigarstring = "5M1N5M"
+        a.query_sequence = "A" * 10
+        a.set_tag('MD', "10")
+        self.assertEqual(
+            "A" * 10,
+            a.get_reference_sequence())
+
+        a.cigarstring = "5M3N5M"
+        a.query_sequence = "A" * 10
+        a.set_tag('MD', "10")
+        self.assertEqual(
+            "A" * 10,
+            a.get_reference_sequence())
+
     def testMDTagSoftClipping(self):
         a = self.buildRead()
 
@@ -561,7 +672,7 @@ class TestTags(ReadTest):
         self.assertEqual(
             "A" * 5 + "C" + "A" * 5,
             a.get_reference_sequence())
-        
+
         # all together
         a.cigarstring = "5S5M1D5M1I5M5S"
         a.query_sequence = "G" * 5 + "A" * 16 + "G" * 5
@@ -579,7 +690,7 @@ class TestTags(ReadTest):
         self.assertEqual(
             "AAcAATCAAAAA",
             a.get_reference_sequence())
-        
+
         a.cigarstring = "5S5M2D1I5M5S"
         a.query_sequence = "G" * 5 + "A" * 11 + "G" * 5
         a.set_tag('MD', "2C2^TC5")
@@ -606,7 +717,7 @@ class TestTags(ReadTest):
 
 
 class TestCopy(ReadTest):
-    
+
     def testCopy(self):
         a = self.buildRead()
         b = copy.copy(a)
