@@ -9,6 +9,8 @@ import unittest
 import os
 import shutil
 import sys
+import re
+import copy
 import collections
 import subprocess
 import logging
@@ -23,7 +25,8 @@ from functools import partial
 import pysam
 import pysam.samtools
 from TestUtils import checkBinaryEqual, checkURL, \
-    checkSamtoolsViewEqual, checkFieldEqual, force_str
+    check_samtools_view_equal, checkFieldEqual, force_str, \
+    get_temp_filename
 
 
 DATADIR = "pysam_data"
@@ -49,6 +52,9 @@ class BasicTestBAMFromFetch(unittest.TestCase):
             "rb")
         self.reads = list(self.samfile.fetch())
 
+    def tearDown(self):
+        self.samfile.close()
+
     def testARqname(self):
         self.assertEqual(
             self.reads[0].query_name,
@@ -261,9 +267,6 @@ class BasicTestBAMFromFetch(unittest.TestCase):
         self.assertEqual(self.reads[0].opt("XT"), "U")
         self.assertEqual(self.reads[1].opt("XT"), "R")
 
-    def tearDown(self):
-        self.samfile.close()
-
 
 class BasicTestSAMFromFetch(BasicTestBAMFromFetch):
 
@@ -426,42 +429,42 @@ class TestIO(unittest.TestCase):
         The *checkf* is used to determine if the files are
         equal.
         '''
-        infile = pysam.AlignmentFile(
-            os.path.join(DATADIR, input_filename),
-            input_mode)
 
-        if "b" in input_mode:
-            self.assertTrue(infile.is_bam)
-            self.assertFalse(infile.is_cram)
-        elif "c" in input_mode:
-            self.assertFalse(infile.is_bam)
-            self.assertTrue(infile.is_cram)
-        else:
-            self.assertFalse(infile.is_cram)
-            self.assertFalse(infile.is_bam)
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, input_filename),
+                input_mode) as infile:
 
-        if use_template:
-            outfile = pysam.AlignmentFile(
-                output_filename,
-                output_mode,
-                reference_filename=sequence_filename,
-                template=infile)
-        else:
-            outfile = pysam.AlignmentFile(
-                output_filename,
-                output_mode,
-                reference_names=infile.references,
-                reference_lengths=infile.lengths,
-                reference_filename=sequence_filename,
-                add_sq_text=False)
+            if "b" in input_mode:
+                self.assertTrue(infile.is_bam)
+                self.assertFalse(infile.is_cram)
+            elif "c" in input_mode:
+                self.assertFalse(infile.is_bam)
+                self.assertTrue(infile.is_cram)
+            else:
+                self.assertFalse(infile.is_cram)
+                self.assertFalse(infile.is_bam)
 
-        iter = infile.fetch()
+            if use_template:
+                outfile = pysam.AlignmentFile(
+                    output_filename,
+                    output_mode,
+                    reference_filename=sequence_filename,
+                    template=infile)
+            else:
+                outfile = pysam.AlignmentFile(
+                    output_filename,
+                    output_mode,
+                    reference_names=infile.references,
+                    reference_lengths=infile.lengths,
+                    reference_filename=sequence_filename,
+                    add_sq_text=False)
 
-        for x in iter:
-            outfile.write(x)
+            iter = infile.fetch()
 
-        infile.close()
-        outfile.close()
+            for x in iter:
+                outfile.write(x)
+
+            outfile.close()
 
         self.assertTrue(checkf(
             os.path.join(DATADIR, reference_filename),
@@ -490,7 +493,7 @@ class TestIO(unittest.TestCase):
                        "tmp_ex2.cram",
                        "rc", "wc",
                        sequence_filename="pysam_data/ex1.fa",
-                       checkf=checkSamtoolsViewEqual)
+                       checkf=check_samtools_view_equal)
 
     def testSAM2BAM(self):
         self.checkEcho("ex2.sam",
@@ -512,7 +515,7 @@ class TestIO(unittest.TestCase):
                        "rb", "wc",
                        sequence_filename="pysam_data/ex1.fa",
                        checkf=partial(
-                           checkSamtoolsViewEqual,
+                           check_samtools_view_equal,
                            without_header=True))
 
     def testCRAM2BAM(self):
@@ -523,7 +526,7 @@ class TestIO(unittest.TestCase):
                        "rc", "wb",
                        sequence_filename="pysam_data/ex1.fa",
                        checkf=partial(
-                           checkSamtoolsViewEqual,
+                           check_samtools_view_equal,
                            without_header=True))
 
     def testSAM2CRAM(self):
@@ -533,7 +536,7 @@ class TestIO(unittest.TestCase):
                        "r", "wc",
                        sequence_filename="pysam_data/ex1.fa",
                        checkf=partial(
-                           checkSamtoolsViewEqual,
+                           check_samtools_view_equal,
                            without_header=True))
 
     def testCRAM2SAM(self):
@@ -543,7 +546,7 @@ class TestIO(unittest.TestCase):
                        "rc", "wh",
                        sequence_filename="pysam_data/ex1.fa",
                        checkf=partial(
-                           checkSamtoolsViewEqual,
+                           check_samtools_view_equal,
                            without_header=True))
 
     # Disabled - should work, files are not binary equal, but are
@@ -858,12 +861,18 @@ class TestIteratorRowBAM(unittest.TestCase):
 
     filename = os.path.join(DATADIR, "ex2.bam")
     mode = "rb"
+    reference_filename = None
 
     def setUp(self):
         self.samfile = pysam.AlignmentFile(
-            self.filename, self.mode,
+            self.filename,
+            self.mode,
+            reference_filename=self.reference_filename,
         )
 
+    def tearDown(self):
+        self.samfile.close()
+
     def checkRange(self, rnge):
         '''compare results from iterator with those from samtools.'''
         ps = list(self.samfile.fetch(region=rnge))
@@ -911,9 +920,6 @@ class TestIteratorRowBAM(unittest.TestCase):
                 self.checkRange("%s:%i-%i" %
                                 (contig, start, start + 90))
 
-    def tearDown(self):
-        self.samfile.close()
-
 
 class TestIteratorRowAllBAM(unittest.TestCase):
 
@@ -1034,9 +1040,9 @@ class TestIteratorRowCRAM(TestIteratorRowBAM):
     mode = "rc"
 
 
-class TestIteratorRowCRAM(TestIteratorRowBAM):
-    filename = os.path.join(DATADIR, "ex2.cram")
-    mode = "rc"
+class TestIteratorRowCRAMWithReferenceFilename(TestIteratorRowCRAM):
+    reference_filename = os.path.join(DATADIR, "ex1.fa")
+
 
 ##########################################################
 ##########################################################
@@ -1392,6 +1398,72 @@ class TestHeader1000Genomes(unittest.TestCase):
         self.assertTrue(data)
 
 
+class TestHeaderWriteRead(unittest.TestCase):
+    header = {'SQ': [{'LN': 1575, 'SN': 'chr1'},
+                     {'LN': 1584, 'SN': 'chr2'}],
+              'RG': [{'LB': 'SC_1', 'ID': 'L1', 'SM': 'NA12891',
+                      'PU': 'SC_1_10', "CN": "name:with:colon"},
+                     {'LB': 'SC_2', 'ID': 'L2', 'SM': 'NA12891',
+                      'PU': 'SC_2_12', "CN": "name:with:colon"}],
+              'PG': [{'ID': 'P1', 'VN': '1.0', 'CL': 'tool'},
+                     {'ID': 'P2', 'VN': '1.1', 'CL': 'tool with in option -R a\tb',
+                      'PP': 'P1'}],
+              'HD': {'VN': '1.0'},
+              'CO': ['this is a comment', 'this is another comment'],
+    }
+
+    def compare_headers(self, a, b):
+        '''compare two headers a and b.
+
+        Ignore M5 and UR field as they are set application specific.
+        '''
+        for ak, av in a.items():
+            self.assertTrue(ak in b, "key '%s' not in '%s' " % (ak, b))
+            self.assertEqual(
+                len(av), len(b[ak]),
+                "unequal number of entries for key {}: {} vs {}"
+                .format(ak, av, b[ak]))
+
+            for row_a, row_b in zip(av, b[ak]):
+                if isinstance(row_b, dict):
+                    for x in ["M5", "UR"]:
+                        try:
+                            del row_b[x]
+                        except KeyError:
+                            pass
+                self.assertEqual(row_a, row_b)
+
+    def check_read_write(self, flag_write, header):
+
+        fn = get_temp_filename()
+        with pysam.AlignmentFile(
+                fn,
+                flag_write,
+                header=header,
+                reference_filename="pysam_data/ex1.fa") as outf:
+            a = pysam.AlignedSegment()
+            a.query_name = "abc"
+            outf.write(a)
+
+        with pysam.AlignmentFile(fn) as inf:
+            read_header = inf.header
+
+        os.unlink(fn)
+        self.compare_headers(header, read_header)
+
+    def test_SAM(self):
+        self.check_read_write("wh", self.header)
+
+    def test_BAM(self):
+        self.check_read_write("wb", self.header)
+
+    def test_CRAM(self):
+        header = copy.copy(self.header)
+        # for CRAM, \t needs to be quoted:
+        header['PG'][1]['CL'] = re.sub(r"\t", r"\\\\t", header['PG'][1]['CL'])
+        self.check_read_write("wc", header)
+
+
 class TestUnmappedReads(unittest.TestCase):
 
     # TODO
@@ -1840,40 +1912,54 @@ class TestBTagBam(TestBTagSam):
     filename = os.path.join(DATADIR, 'example_btag.bam')
 
 
-class TestDoubleFetch(unittest.TestCase):
-
+class TestDoubleFetchBAM(unittest.TestCase):
     '''check if two iterators on the same bamfile are independent.'''
 
     filename = os.path.join(DATADIR, 'ex1.bam')
+    mode = "rb"
 
     def testDoubleFetch(self):
 
-        samfile1 = pysam.AlignmentFile(self.filename, 'rb')
-
-        for a, b in zip(samfile1.fetch(multiple_iterators=True),
-                        samfile1.fetch(multiple_iterators=True)):
-            self.assertEqual(a.compare(b), 0)
+        with pysam.AlignmentFile(self.filename, self.mode) as samfile1:
+            for a, b in zip(samfile1.fetch(multiple_iterators=True),
+                            samfile1.fetch(multiple_iterators=True)):
+                self.assertEqual(a.compare(b), 0)
 
     def testDoubleFetchWithRegion(self):
 
-        samfile1 = pysam.AlignmentFile(self.filename, 'rb')
-        chr, start, stop = 'chr1', 200, 3000000
-        # just making sure the test has something to catch
-        self.assertTrue(len(list(samfile1.fetch(chr, start, stop))) > 0)
+        with pysam.AlignmentFile(self.filename, self.mode) as samfile1:
+            contig, start, stop = 'chr1', 200, 3000000
+            # just making sure the test has something to catch
+            self.assertTrue(len(list(samfile1.fetch(contig, start, stop))) > 0)
 
-        for a, b in zip(samfile1.fetch(chr, start, stop),
-                        samfile1.fetch(chr, start, stop,
-                                       multiple_iterators=True)):
-            self.assertEqual(a.compare(b), 0)
+            # see Issue #293
+            # The following fails for CRAM files, but works for BAM
+            # files when the first is multiple_iterators=False:
+            for a, b in zip(samfile1.fetch(contig, start, stop,
+                                           multiple_iterators=True),
+                            samfile1.fetch(contig, start, stop,
+                                           multiple_iterators=True)):
+                self.assertEqual(a.compare(b), 0)
 
     def testDoubleFetchUntilEOF(self):
 
-        samfile1 = pysam.AlignmentFile(self.filename, 'rb')
+        with pysam.AlignmentFile(self.filename, self.mode) as samfile1:
 
-        for a, b in zip(samfile1.fetch(until_eof=True),
-                        samfile1.fetch(until_eof=True,
-                                       multiple_iterators=True)):
-            self.assertEqual(a.compare(b), 0)
+            for a, b in zip(samfile1.fetch(until_eof=True),
+                            samfile1.fetch(until_eof=True,
+                                           multiple_iterators=True)):
+                self.assertEqual(a.compare(b), 0)
+
+
+class TestDoubleFetchCRAM(TestDoubleFetchBAM):
+    filename = os.path.join(DATADIR, 'ex2.cram')
+    mode = "rc"
+
+
+class TestDoubleFetchCRAMWithReference(TestDoubleFetchBAM):
+    filename = os.path.join(DATADIR, 'ex2.cram')
+    mode = "rc"
+    reference_filename = os.path.join(DATADIR, 'ex1.fa')
 
 
 class TestRemoteFileFTP(unittest.TestCase):
@@ -1926,10 +2012,11 @@ class TestRemoteFileHTTP(unittest.TestCase):
         if not checkURL(self.url):
             return
 
-        samfile = pysam.AlignmentFile(self.url, "rb")
-        result = list(samfile.fetch(region=self.region))
-        samfile_local = pysam.AlignmentFile(self.local, "rb")
-        ref = list(samfile_local.fetch(region=self.region))
+        with pysam.AlignmentFile(self.url, "rb") as samfile:
+            result = list(samfile.fetch(region=self.region))
+
+        with pysam.AlignmentFile(self.local, "rb") as samfile_local:
+            ref = list(samfile_local.fetch(region=self.region))
 
         self.assertEqual(len(ref), len(result))
         for x, y in zip(result, ref):
@@ -1939,10 +2026,11 @@ class TestRemoteFileHTTP(unittest.TestCase):
         if not checkURL(self.url):
             return
 
-        samfile = pysam.AlignmentFile(self.url, "rb")
-        result = list(samfile.fetch())
-        samfile_local = pysam.AlignmentFile(self.local, "rb")
-        ref = list(samfile_local.fetch())
+        with pysam.AlignmentFile(self.url, "rb") as samfile:
+            result = list(samfile.fetch())
+
+        with pysam.AlignmentFile(self.local, "rb") as samfile_local:
+            ref = list(samfile_local.fetch())
 
         self.assertEqual(len(ref), len(result))
         for x, y in zip(result, ref):
@@ -2009,6 +2097,10 @@ class TestPileup(unittest.TestCase):
         self.samfile = pysam.AlignmentFile(self.samfilename)
         self.fastafile = pysam.Fastafile(self.fastafilename)
 
+    def tearDown(self):
+        self.samfile.close()
+        self.fastafile.close()
+
     def checkEqual(self, references, iterator):
 
         for x, column in enumerate(iterator):
@@ -2070,6 +2162,10 @@ class TestCountCoverage(unittest.TestCase):
         samfile.close()
         pysam.samtools.index("test_count_coverage_read_all.bam")
 
+    def tearDown(self):
+        self.samfile.close()
+        self.fastafile.close()
+
     def count_coverage_python(self, bam, chrom, start, stop,
                               read_callback,
                               quality_threshold=15):
@@ -2161,23 +2257,26 @@ class TestCountCoverage(unittest.TestCase):
         self.assertEqual(fast_counts[3], manual_counts[3])
 
     def test_count_coverage_read_all(self):
-        samfile = pysam.AlignmentFile("test_count_coverage_read_all.bam")
+
         chrom = 'chr1'
         start = 0
         stop = 2000
 
         def filter(read):
             return not (read.flag & (0x4 | 0x100 | 0x200 | 0x400))
-        fast_counts = samfile.count_coverage(
-            chrom, start, stop,
-            read_callback='all',
-            #read_callback = lambda read: ~(read.flag & (0x4 | 0x100 | 0x200 | 0x400)),
-            quality_threshold=0)
-        manual_counts = samfile.count_coverage(
-            chrom, start, stop,
-            read_callback=lambda read: not(
-                read.flag & (0x4 | 0x100 | 0x200 | 0x400)),
-            quality_threshold=0)
+
+        with pysam.AlignmentFile("test_count_coverage_read_all.bam") as samfile:
+
+            fast_counts = samfile.count_coverage(
+                chrom, start, stop,
+                read_callback='all',
+                #read_callback = lambda read: ~(read.flag & (0x4 | 0x100 | 0x200 | 0x400)),
+                quality_threshold=0)
+            manual_counts = samfile.count_coverage(
+                chrom, start, stop,
+                read_callback=lambda read: not(
+                    read.flag & (0x4 | 0x100 | 0x200 | 0x400)),
+                quality_threshold=0)
 
         os.unlink("test_count_coverage_read_all.bam")
         os.unlink("test_count_coverage_read_all.bam.bai")
@@ -2202,18 +2301,20 @@ class TestCountCoverage(unittest.TestCase):
             samfile.write(read)
         samfile.close()
         pysam.samtools.index("test_count_coverage_nofilter.bam")
-        samfile = pysam.AlignmentFile("test_count_coverage_nofilter.bam")
         chr = 'chr1'
         start = 0
         stop = 2000
-        fast_counts = samfile.count_coverage(chr, start, stop,
-                                             read_callback='nofilter',
-                                             quality_threshold=0)
 
-        manual_counts = self.count_coverage_python(samfile, chr, start, stop,
-                                                   read_callback=lambda x: True,
-                                                   quality_threshold=0)
-        samfile.close()
+        with pysam.AlignmentFile("test_count_coverage_nofilter.bam") as samfile:
+
+            fast_counts = samfile.count_coverage(chr, start, stop,
+                                                 read_callback='nofilter',
+                                                 quality_threshold=0)
+
+            manual_counts = self.count_coverage_python(samfile, chr, start, stop,
+                                                       read_callback=lambda x: True,
+                                                       quality_threshold=0)
+
         os.unlink("test_count_coverage_nofilter.bam")
         os.unlink("test_count_coverage_nofilter.bam.bai")
         self.assertEqual(fast_counts[0], manual_counts[0])
@@ -2223,7 +2324,7 @@ class TestCountCoverage(unittest.TestCase):
 
 
 class TestPileupQueryPosition(unittest.TestCase):
-    
+
     filename = "test_query_position.bam"
 
     def testPileup(self):
@@ -2260,8 +2361,8 @@ class TestLogging(unittest.TestCase):
             log_hand.setFormatter(formatter)
             logger.addHandler(log_hand)
 
-        bam = pysam.AlignmentFile(bamfile, 'rb')
-        cols = bam.pileup()
+        with pysam.AlignmentFile(bamfile, 'rb') as bam:
+            cols = bam.pileup()
         self.assertTrue(True)
 
     def testFail1(self):
@@ -2292,40 +2393,41 @@ class TestAlignmentFileUtilityFunctions(unittest.TestCase):
 
     def testCount(self):
 
-        samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
-                                      "rb")
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "ex1.bam"),
+                "rb") as samfile:
 
-        for contig in ("chr1", "chr2"):
-            for start in range(0, 2000, 100):
-                end = start + 1
-                self.assertEqual(
-                    len(list(samfile.fetch(contig, start, end))),
-                    samfile.count(contig, start, end),
-                    'number mismatch for %s:%i-%i %i != %i' % (
-                        contig, start, end,
+            for contig in ("chr1", "chr2"):
+                for start in range(0, 2000, 100):
+                    end = start + 1
+                    self.assertEqual(
                         len(list(samfile.fetch(contig, start, end))),
-                        samfile.count(contig, start, end)))
+                        samfile.count(contig, start, end),
+                        'number mismatch for %s:%i-%i %i != %i' % (
+                            contig, start, end,
+                            len(list(samfile.fetch(contig, start, end))),
+                            samfile.count(contig, start, end)))
 
-                # test empty intervals
-                self.assertEqual(
-                    len(list(samfile.fetch(contig, start, start))),
-                    samfile.count(contig, start, start),
-                    'number mismatch for %s:%i-%i %i != %i' % (
-                        contig, start, start,
+                    # test empty intervals
+                    self.assertEqual(
                         len(list(samfile.fetch(contig, start, start))),
-                        samfile.count(contig, start, start)))
+                        samfile.count(contig, start, start),
+                        'number mismatch for %s:%i-%i %i != %i' % (
+                            contig, start, start,
+                            len(list(samfile.fetch(contig, start, start))),
+                            samfile.count(contig, start, start)))
 
-                # test half empty intervals
-                self.assertEqual(len(list(samfile.fetch(contig, start))),
-                                 samfile.count(contig, start))
+                    # test half empty intervals
+                    self.assertEqual(len(list(samfile.fetch(contig, start))),
+                                     samfile.count(contig, start))
 
-                self.assertEqual(
-                    len(list(samfile.fetch(contig, start))),
-                    samfile.count(contig, start),
-                    'number mismatch for %s:%i %i != %i' % (
-                        contig, start,
+                    self.assertEqual(
                         len(list(samfile.fetch(contig, start))),
-                        samfile.count(contig, start)))
+                        samfile.count(contig, start),
+                        'number mismatch for %s:%i %i != %i' % (
+                            contig, start,
+                            len(list(samfile.fetch(contig, start))),
+                            samfile.count(contig, start)))
 
     def testMate(self):
         '''test mate access.'''
@@ -2339,35 +2441,35 @@ class TestAlignmentFileUtilityFunctions(unittest.TestCase):
         for x in readnames:
             counts[x] += 1
 
-        samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
-                                      "rb")
+        with pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
+                                 "rb") as samfile:
 
-        for read in samfile.fetch():
-            if not read.is_paired:
-                self.assertRaises(ValueError, samfile.mate, read)
-            elif read.mate_is_unmapped:
-                self.assertRaises(ValueError, samfile.mate, read)
-            else:
-                if counts[read.query_name] == 1:
+            for read in samfile.fetch():
+                if not read.is_paired:
+                    self.assertRaises(ValueError, samfile.mate, read)
+                elif read.mate_is_unmapped:
                     self.assertRaises(ValueError, samfile.mate, read)
                 else:
-                    mate = samfile.mate(read)
-                    self.assertEqual(read.query_name, mate.query_name)
-                    self.assertEqual(read.is_read1, mate.is_read2)
-                    self.assertEqual(read.is_read2, mate.is_read1)
-                    self.assertEqual(
-                        read.reference_start, mate.next_reference_start)
-                    self.assertEqual(
-                        read.next_reference_start, mate.reference_start)
+                    if counts[read.query_name] == 1:
+                        self.assertRaises(ValueError, samfile.mate, read)
+                    else:
+                        mate = samfile.mate(read)
+                        self.assertEqual(read.query_name, mate.query_name)
+                        self.assertEqual(read.is_read1, mate.is_read2)
+                        self.assertEqual(read.is_read2, mate.is_read1)
+                        self.assertEqual(
+                            read.reference_start, mate.next_reference_start)
+                        self.assertEqual(
+                            read.next_reference_start, mate.reference_start)
 
     def testIndexStats(self):
         '''test if total number of mapped/unmapped reads is correct.'''
 
-        samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
-                                      "rb")
-        self.assertEqual(samfile.mapped, 3235)
-        self.assertEqual(samfile.unmapped, 35)
-        self.assertEqual(samfile.nocoordinate, 0)
+        with pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
+                                 "rb") as samfile:
+            self.assertEqual(samfile.mapped, 3235)
+            self.assertEqual(samfile.unmapped, 35)
+            self.assertEqual(samfile.nocoordinate, 0)
 
 
 class TestMappedUnmapped(unittest.TestCase):
@@ -2452,26 +2554,29 @@ class TestAlignmentFileIndex(unittest.TestCase):
 class TestExplicitIndex(unittest.TestCase):
 
     def testExplicitIndexBAM(self):
-        samfile = pysam.AlignmentFile(
-            os.path.join(DATADIR, "explicit_index.bam"),
-            "rb",
-            filepath_index=os.path.join(DATADIR, 'ex1.bam.bai'))
-                                      
-        samfile.fetch("chr1")
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "explicit_index.bam"),
+                "rb",
+                filepath_index=os.path.join(DATADIR, 'ex1.bam.bai')) as samfile:
+            samfile.fetch("chr1")
 
     def testExplicitIndexCRAM(self):
-        samfile = pysam.AlignmentFile(
-            os.path.join(DATADIR, "explicit_index.cram"),
-            "rc",
-            filepath_index=os.path.join(DATADIR, 'ex1.cram.crai'))
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "explicit_index.cram"),
+                "rc",
+                filepath_index=os.path.join(DATADIR, 'ex1.cram.crai')) as samfile:
+            samfile.fetch("chr1")
 
     def testRemoteExplicitIndexBAM(self):
-        samfile = pysam.AlignmentFile(
-            "http://genserv.anat.ox.ac.uk/downloads/pysam/test/noindex.bam",
-            "rb",
-            filepath_index=os.path.join(DATADIR, 'ex1.bam.bai'))
+        if not checkURL(
+                "http://genserv.anat.ox.ac.uk/downloads/pysam/test/noindex.bam"):
+            return
 
-        samfile.fetch("chr1")
+        with pysam.AlignmentFile(
+                "http://genserv.anat.ox.ac.uk/downloads/pysam/test/noindex.bam",
+                "rb",
+                filepath_index=os.path.join(DATADIR, 'ex1.bam.bai')) as samfile:
+            samfile.fetch("chr1")
 
 
 class TestVerbosity(unittest.TestCase):
