@@ -6,6 +6,7 @@ and data files located there.
 '''
 
 import pysam
+import pysam.samtools
 import unittest
 import os
 import shutil
@@ -14,12 +15,8 @@ import collections
 import subprocess
 import logging
 import array
-from TestUtils import checkBinaryEqual, checkURL
+from TestUtils import checkBinaryEqual, checkURL, force_str
 
-IS_PYTHON3 = sys.version_info[0] >= 3
-
-SAMTOOLS = "samtools"
-WORKDIR = "pysam_test_work"
 DATADIR = "pysam_data"
 
 
@@ -755,23 +752,34 @@ class TestIteratorRow(unittest.TestCase):
     def checkRange(self, rnge):
         '''compare results from iterator with those from samtools.'''
         ps = list(self.samfile.fetch(region=rnge))
-        sa = list(pysam.view(os.path.join(DATADIR, "ex1.bam"),
-                             rnge,
-                             raw=True))
-        self.assertEqual(len(ps), len(
-            sa), "unequal number of results for range %s: %i != %i" % (rnge, len(ps), len(sa)))
+        sa = force_str(
+            pysam.samtools.view(
+                os.path.join(DATADIR, "ex1.bam"),
+                rnge,
+                raw=True)).splitlines(True)
+        self.assertEqual(
+            len(ps), len(sa),
+            "unequal number of results for range %s: %i != %i" %
+            (rnge, len(ps), len(sa)))
         # check if the same reads are returned and in the same order
         for line, (a, b) in enumerate(list(zip(ps, sa))):
             d = b.split("\t")
             self.assertEqual(
-                a.qname, d[0], "line %i: read id mismatch: %s != %s" % (line, a.rname, d[0]))
-            self.assertEqual(a.pos, int(d[3]) - 1, "line %i: read position mismatch: %s != %s, \n%s\n%s\n" %
-                             (line, a.pos, int(d[3]) - 1,
-                              str(a), str(d)))
+                a.qname, d[0],
+                "line %i: read id mismatch: %s != %s" %
+                (line, a.rname, d[0]))
+            self.assertEqual(
+                a.pos, int(d[3]) - 1,
+                "line %i: read position mismatch: %s != %s, "
+                "\n%s\n%s\n" %
+                (line, a.pos, int(d[3]) - 1,
+                 str(a), str(d)))
             qual = d[10]
-            self.assertEqual(a.qual, qual, "line %i: quality mismatch: %s != %s, \n%s\n%s\n" %
-                             (line, a.qual, qual,
-                              str(a), str(d)))
+            self.assertEqual(
+                a.qual, qual,
+                "line %i: quality mismatch: %s != %s, \n%s\n%s\n" %
+                (line, a.qual, qual,
+                 str(a), str(d)))
 
     def testIteratePerContig(self):
         '''check random access per contig'''
@@ -798,8 +806,11 @@ class TestIteratorRowAll(unittest.TestCase):
     def testIterate(self):
         '''compare results from iterator with those from samtools.'''
         ps = list(self.samfile.fetch())
-        sa = list(pysam.view(os.path.join(DATADIR, "ex1.bam"),
-                             raw=True))
+        sa = force_str(
+            pysam.samtools.view(
+                os.path.join(DATADIR, "ex1.bam"),
+                raw=True)).splitlines(True)
+
         self.assertEqual(
             len(ps), len(sa), "unequal number of results: %i != %i" % (len(ps), len(sa)))
         # check if the same reads are returned
@@ -853,7 +864,8 @@ class TestIteratorColumn(unittest.TestCase):
 
     def testIterateRanges(self):
         '''check random access per range'''
-        for contig, length in zip(self.samfile.references, self.samfile.lengths):
+        for contig, length in zip(
+                self.samfile.references, self.samfile.lengths):
             for start in range(1, length, 90):
                 # this includes empty ranges
                 self.checkRange(contig, start, start + 90)
@@ -1661,7 +1673,7 @@ class TestRemoteFileFTP(unittest.TestCase):
         if not checkURL(self.url):
             return
 
-        result = pysam.view(self.url, self.region)
+        result = pysam.samtools.view(self.url, self.region)
         self.assertEqual(len(result), 36)
 
     def testFTPFetch(self):
@@ -1687,7 +1699,8 @@ class TestRemoteFileHTTP(unittest.TestCase):
         samfile_local = pysam.Samfile(self.local, "rb")
         ref = list(samfile_local.fetch(region=self.region))
 
-        result = pysam.view(self.url, self.region)
+        result = pysam.samtools.view(
+            self.url, self.region).splitlines(True)
         self.assertEqual(len(result), len(ref))
 
     def testFetch(self):
@@ -1786,20 +1799,22 @@ class TestPileup(unittest.TestCase):
             self.assertEqual(int(pos) - 1, column.pos)
 
     def testSamtoolsStepper(self):
-        refs = pysam.mpileup(
-            "-f", self.fastafilename,
-            self.samfilename)
+        refs = force_str(
+            pysam.samtools.mpileup(
+                "-f", self.fastafilename,
+                self.samfilename)).splitlines(True)
         iterator = self.samfile.pileup(
             stepper="samtools",
             fastafile=self.fastafile)
         self.checkEqual(refs, iterator)
 
     def testAllStepper(self):
-        refs = pysam.mpileup(
-            "-f", self.fastafilename,
-            "-A", "-B",
-            self.samfilename)
-
+        refs = force_str(
+            pysam.samtools.mpileup(
+                "-f", self.fastafilename,
+                "-A", "-B",
+                self.samfilename)).splitlines(True)
+            
         iterator = self.samfile.pileup(
             stepper="all",
             fastafile=self.fastafile)
@@ -1937,14 +1952,14 @@ class TestSamtoolsProxy(unittest.TestCase):
     '''tests for sanity checking access to samtools functions.'''
 
     def testIndex(self):
-        self.assertRaises(IOError, pysam.index, "missing_file")
+        self.assertRaises(IOError, pysam.samtools.index, "missing_file")
 
     def testView(self):
         # note that view still echos "open: No such file or directory"
-        self.assertRaises(pysam.SamtoolsError, pysam.view, "missing_file")
+        self.assertRaises(pysam.SamtoolsError, pysam.samtools.view, "missing_file")
 
     def testSort(self):
-        self.assertRaises(pysam.SamtoolsError, pysam.sort, "missing_file")
+        self.assertRaises(pysam.SamtoolsError, pysam.samtools.sort, "missing_file")
 
 
 class TestSamfileIndex(unittest.TestCase):
