@@ -270,6 +270,9 @@ class TestIterationWithoutComments(IterationTest):
         IterationTest.setUp(self)
         self.tabix = pysam.TabixFile(self.filename)
 
+    def tearDown(self):
+        self.tabix.close()
+
     def testRegionStrings(self):
         """test if access with various region strings
         works"""
@@ -351,7 +354,7 @@ class TestIterationWithoutComments(IterationTest):
         self.tabix.fetch("chr1", 100, 100)
 
     def testGetContigs(self):
-        self.assertEqual(sorted(self.tabix.contigs), [b"chr1", b"chr2"])
+        self.assertEqual(sorted(self.tabix.contigs), ["chr1", "chr2"])
         # check that contigs is read-only
         self.assertRaises(
             AttributeError, setattr, self.tabix, "contigs", ["chr1", "chr2"])
@@ -374,13 +377,10 @@ class TestIterationWithoutComments(IterationTest):
             # opens any tabix file
             with pysam.TabixFile(self.filename) as inf:
                 pass
-            
+
         for i in range(1000):
             func1()
 
-    def tearDown(self):
-        self.tabix.close()
-
 
 class TestIterationWithComments(TestIterationWithoutComments):
 
@@ -405,6 +405,9 @@ class TestParser(unittest.TestCase):
         self.tabix = pysam.TabixFile(self.filename)
         self.compare = loadAndConvert(self.filename)
 
+    def tearDown(self):
+        self.tabix.close()
+
     def testRead(self):
 
         for x, r in enumerate(self.tabix.fetch(parser=pysam.asTuple())):
@@ -505,6 +508,36 @@ class TestParser(unittest.TestCase):
         self.assertEqual(a, b)
 
 
+class TestGTF(TestParser):
+
+    def testRead(self):
+
+        for x, r in enumerate(self.tabix.fetch(parser=pysam.asGTF())):
+            c = self.compare[x]
+            self.assertEqual(len(c), len(r))
+            self.assertEqual(list(c), list(r))
+            self.assertEqual(c, str(r).split("\t"))
+            self.assertTrue(r.gene_id.startswith("ENSG"))
+            if r.feature != 'gene':
+                self.assertTrue(r.transcript_id.startswith("ENST"))
+            self.assertEqual(c[0], r.contig)
+            self.assertEqual("\t".join(map(str, c)),
+                             str(r))
+
+    def testSetting(self):
+
+        for r in self.tabix.fetch(parser=pysam.asGTF()):
+            r.contig = r.contig + "_test"          
+            r.source = r.source + "_test"
+            r.feature = r.feature + "_test"
+            r.start += 10
+            r.end += 10
+            r.score = 20
+            r.strand = "+"
+            r.frame = 0
+            r.attributes = 'gene_id "0001";'
+
+
 class TestIterators(unittest.TestCase):
 
     filename = os.path.join(DATADIR, "example.gtf.gz")
@@ -522,6 +555,10 @@ class TestIterators(unittest.TestCase):
              open(self.tmpfilename_uncompressed, "wb") as outfile:
             outfile.write(infile.read())
 
+    def tearDown(self):
+        self.tabix.close()
+        os.unlink(self.tmpfilename_uncompressed)
+
     def open(self):
 
         if self.is_compressed:
@@ -566,9 +603,6 @@ class TestIterators(unittest.TestCase):
         # Not implemented
         # self.assertRaises(ValueError, i.next)
 
-    def tearUp(self):
-        os.unlink(self.tmpfilename_uncompressed)
-
 
 class TestIteratorsGenericCompressed(TestIterators):
     is_compressed = True
@@ -584,23 +618,6 @@ class TestIteratorsFileUncompressed(TestIterators):
     is_compressed = False
 
 
-class TestGTF(TestParser):
-
-    def testRead(self):
-
-        for x, r in enumerate(self.tabix.fetch(parser=pysam.asGTF())):
-            c = self.compare[x]
-            self.assertEqual(len(c), len(r))
-            self.assertEqual(list(c), list(r))
-            self.assertEqual(c, str(r).split("\t"))
-            self.assertTrue(r.gene_id.startswith("ENSG"))
-            if r.feature != 'gene':
-                self.assertTrue(r.transcript_id.startswith("ENST"))
-            self.assertEqual(c[0], r.contig)
-            self.assertEqual("\t".join(map(str, c)),
-                             str(r))
-
-
 class TestIterationMalformattedGTFFiles(unittest.TestCase):
 
     '''test reading from malformatted gtf files.'''
@@ -638,6 +655,9 @@ class TestBed(unittest.TestCase):
         self.tabix = pysam.TabixFile(self.filename)
         self.compare = loadAndConvert(self.filename)
 
+    def tearDown(self):
+        self.tabix.close()
+
     def testRead(self):
 
         for x, r in enumerate(self.tabix.fetch(parser=pysam.asBed())):
@@ -670,9 +690,6 @@ class TestBed(unittest.TestCase):
             self.assertEqual(int(c[2]) + 1, r.end)
             self.assertEqual(str(int(c[2]) + 1), r[2])
 
-    def tearDown(self):
-        self.tabix.close()
-
 
 class TestVCF(unittest.TestCase):
 
@@ -736,6 +753,9 @@ class TestVCFFromTabix(TestVCF):
         self.tabix = pysam.TabixFile(self.tmpfilename + ".gz")
         self.compare = loadAndConvert(self.filename)
 
+    def tearDown(self):
+        self.tabix.close()
+
     def testRead(self):
 
         ncolumns = len(self.columns)
@@ -804,9 +824,6 @@ class TestVCFFromTabix(TestVCF):
                 c[ncolumns + y] = "test_%i" % y
                 r[y] = "test_%i" % y
                 self.assertEqual(c[ncolumns + y], r[y])
-                
-    def tearDown(self):
-        self.tabix.close()
 
 
 class TestVCFFromVCF(TestVCF):
@@ -843,6 +860,9 @@ class TestVCFFromVCF(TestVCF):
         self.vcf = pysam.VCF()
         self.compare = loadAndConvert(self.filename, encode=False)
 
+    def tearDown(self):
+        self.vcf.close()
+
     def testConnecting(self):
 
         fn = os.path.basename(self.filename)
@@ -856,15 +876,25 @@ class TestVCFFromVCF(TestVCF):
 
     def get_iterator(self):
 
-        f = open(self.filename)
-        fn = os.path.basename(self.filename)
+        with open(self.filename) as f:
+            fn = os.path.basename(self.filename)
 
-        for x, msg in self.fail_on_opening:
-            if "%i.vcf" % x == fn:
-                self.assertRaises(ValueError, self.vcf.parse, f)
-                return
+            for x, msg in self.fail_on_opening:
+                if "%i.vcf" % x == fn:
+                    self.assertRaises(ValueError, self.vcf.parse, f)
+                    return
 
-        return self.vcf.parse(f)
+            for vcf_code, msg in self.fail_on_parsing:
+                if "%i.vcf" % vcf_code == fn:
+                    self.assertRaises((ValueError,
+                                       AssertionError),
+                                      list, self.vcf.parse(f))
+                    return
+                # python 2.7
+                # self.assertRaisesRegexp(
+                # ValueError, re.compile(msg), self.vcf.parse, f)
+
+            return list(self.vcf.parse(f))
 
     def get_field_value(self, record, field):
         return record[field]
@@ -1063,6 +1093,8 @@ class TestVCFFromVariantFile(TestVCFFromVCF):
     missing_value = None
     missing_quality = None
 
+    vcf = None
+
     def filter2value(self, r, v):
         if r == "PASS":
             return ["PASS"], list(v)
@@ -1104,9 +1136,14 @@ class TestVCFFromVariantFile(TestVCFFromVCF):
         TestVCF.setUp(self)
         self.compare = loadAndConvert(self.filename, encode=False)
 
+    def tearDown(self):
+        if self.vcf:
+            self.vcf.close()
+        self.vcf = None
+
     def get_iterator(self):
-        vcf = pysam.VariantFile(self.filename)
-        return vcf.fetch()
+        self.vcf = pysam.VariantFile(self.filename)
+        return self.vcf.fetch()
 
     def get_field_value(self, record, field):
         return getattr(record, field)
@@ -1124,11 +1161,22 @@ class TestRemoteFileHTTP(unittest.TestCase):
     local = os.path.join(DATADIR, "example.gtf.gz")
 
     def setUp(self):
+        if not checkURL(self.url):
+            self.remote_file = None
+            return
+
         self.remote_file = pysam.TabixFile(self.url, "r")
         self.local_file = pysam.TabixFile(self.local, "r")
 
+    def tearDown(self):
+        if self.remote_file is None:
+            return
+
+        self.remote_file.close()
+        self.local_file.close()
+
     def testFetchAll(self):
-        if not checkURL(self.url):
+        if self.remote_file is None:
             return
 
         remote_result = list(self.remote_file.fetch())
@@ -1139,16 +1187,15 @@ class TestRemoteFileHTTP(unittest.TestCase):
             self.assertEqual(x, y)
 
     def testHeader(self):
+        if self.remote_file is None:
+            return
+
         self.assertEqual(list(self.local_file.header), [])
         self.assertRaises(AttributeError,
                           getattr,
                           self.remote_file,
                           "header")
 
-    def tearDown(self):
-        self.remote_file.close()
-        self.local_file.close()
-
 
 class TestIndexArgument(unittest.TestCase):
 
@@ -1163,13 +1210,11 @@ class TestIndexArgument(unittest.TestCase):
         shutil.copyfile(self.index_src, self.index_dst)
 
         with pysam.TabixFile(
-            self.filename_src, "r", index=self.index_src) as \
-            same_basename_file:
+                self.filename_src, "r", index=self.index_src) as same_basename_file:
             same_basename_results = list(same_basename_file.fetch())
 
         with pysam.TabixFile(
-            self.filename_dst, "r", index=self.index_dst) as \
-            diff_index_file:
+                self.filename_dst, "r", index=self.index_dst) as diff_index_file:
             diff_index_result = list(diff_index_file.fetch())
 
         self.assertEqual(len(same_basename_results), len(diff_index_result))
@@ -1263,7 +1308,7 @@ class TestMultipleIterators(unittest.TestCase):
 
     def testDoubleFetch(self):
 
-        with pysam.TabixFile(self.filename) as f: 
+        with pysam.TabixFile(self.filename) as f:
 
             for a, b in zip(f.fetch(multiple_iterators=True),
                             f.fetch(multiple_iterators=True)):
